#!/usr/bin/env python
#
# Convert MLS exported CSV to a KML compatible with Google Earth and Gooogle Maps
#	by David Ostler at June 28, 2007.  Modified by Gregory Moody October 10, 2007.
#	
#
# Description:
#
#   Imports Membership.csv file exported from MLS and converts to a KML
#	file compatible with Google Earh and Google Maps.  Use only the
#       Membership.csv from MLS and not from ward website.  This modified
#       version will display seperate Icons by Priesthood of HoH in specific folders.
#
# How it works:
#
#   This is a Python script that parses the MLS Ward Membership dump in
#   the CSV format (Comma Separated Values) and generates a KML file.
#   Only the "Head of Household" Name and address is imported to avoid 
#	multiple entries for families.
#
# Instructions:
#
#   1.	Export you ward membership data from MLS to Membership.csv(default).
#
#      
#   2.	run "mls2GoogleEarthSorted.py" with the name of the .csv file as an argument.
#               This creates a file called Home_wardMap.kml with geocoded addresses.
#               It also saves geoError.txt**  Alternatively, a second argument may be
#               supplied to name the specifc ward.             
#
#   3.	Import the Home_wardMap.kml into Google Earth.  Alternatively you can 
#		upload the Home_wardMap.kml file to a web server and search for the 
#		path in google maps like this "http://some.webserver.com/Home_wardMap.kml"
#		then copy this address into the google maps search field and search.
#		The result will be a location map of everyone in the ward. 
#
#     	**The geoError.txt file is saved out to identify addresses that google
#		could not interpret.  It will also note any incomplete information in the
#		MLS database.  Any address with "DNC" will be ommitted from the list.
#		This is a code a ward may use for people who have asked to be "Do Not Contact"
#		
#		***Google maps may block your ip address from using their services if
#		you run this script too many times or if your list is too long (>1000).  
#
#	4. **Optional**  Get an API key here http://www.google.com/apis/maps/signup.html
#		and put it into the key='' field below example: key='AAAbkdkks82984ikkadlfjjxkkjzi99w0890a0ikk'
# 
#
# 	
# 
#

import sys, csv, re, os, urllib, time


key=''

##If the membershipReformat.csv file already exists we can save time retrieving geo date from this file
 
phoneX={}
latitudeX={}
longitudeX={}
addressX={}
postalX={} 
used=''

if os.path.exists('membershipReformat.csv'):
	print "The membershipReformat.csv file exists.  I can save some time by sucking the geo data from this file"
	memRead=open('membershipReformat.csv')
	memReader=csv.reader(memRead, dialect="excel",quotechar="'", delimiter=";")
	header=memReader.next()
	for each in memReader:
		phoneX[each[15]]=each[2]
		latitudeX[each[15]]=each[13]
		longitudeX[each[15]]=each[14]
		addressX[each[15]]=each[4]
		postalX[each[15]]=each[8]
	memRead.close()







# CSV file as command line argument
try:
	file = sys.argv[1]
except:
	print "usage: %s Membership.csv"%sys.argv[0]
	sys.exit(1)

# Grab ward name parameter if available 
try:
        Ward = sys.argv[2]
except:
        Ward = "Home"

# Open CSV file for reading
try:
	f = open(file)
except:
	print "Can't open CSV file:", file
	sys.exit(1)

# Set and open output file
outfile = 'membershipReformat.csv'
try:
	memReformat = open(outfile, 'w')
except:
	print "Can't open output file for writing:", file
	sys.exit(1)
	

# Send all messages to the output file
#real_stdout = sys.stdout
#sys.stdout = memReformat

# parse file
rows = csv.reader(f)
writer = csv.writer(memReformat,delimiter=';')
writer.writerow(["First Name", "Last Name","Primary Phone", "Secondary Phone", "Street Addreess", "Apartment", "City", "State", "Zip", "Country", "Primary email", "Secondary email", "Birth", "Latitude", "Longitude", "MRN"])
# skip headers (first line)
rows.next()

# loop for all records found except "Head of Household"

myHeader='''<?xml version="1.0" encoding="UTF-8"?>
<kml xmlns="http://earth.google.com/kml/2.0">
<Document>
<name>%s Ward</name>
	<Style id="sh_grn-stars">
		<IconStyle>
			<scale>0.7</scale>
			<Icon>
				<href>http://maps.google.com/mapfiles/kml/paddle/grn-stars.png</href>
			</Icon>
			<hotSpot x="32" y="1" xunits="pixels" yunits="pixels"/>
		</IconStyle>
		<LabelStyle>
			<color>ff00aa00</color>
		</LabelStyle>
	</Style>
	<Style id="sn_grn-stars">
		<IconStyle>
			<scale>0.5</scale>
			<Icon>
				<href>http://maps.google.com/mapfiles/kml/paddle/grn-stars.png</href>
			</Icon>
			<hotSpot x="32" y="1" xunits="pixels" yunits="pixels"/>
		</IconStyle>
		<LabelStyle>
			<color>0000aa00</color>
		</LabelStyle>
	</Style>
	<Style id="sh_blu-stars">
		<IconStyle>
			<scale>0.7</scale>
			<Icon>
				<href>http://maps.google.com/mapfiles/kml/paddle/blu-stars.png</href>
			</Icon>
			<hotSpot x="32" y="1" xunits="pixels" yunits="pixels"/>
		</IconStyle>
		<LabelStyle>
			<color>ffff0000</color>
		</LabelStyle>
	</Style>
	<Style id="sn_blu-stars">
		<IconStyle>
			<scale>0.5</scale>
			<Icon>
				<href>http://maps.google.com/mapfiles/kml/paddle/blu-stars.png</href>
			</Icon>
			<hotSpot x="32" y="1" xunits="pixels" yunits="pixels"/>
		</IconStyle>
		<LabelStyle>
			<color>00ff0000</color>
		</LabelStyle>
	</Style>
	<Style id="sh_wht-stars">
		<IconStyle>
			<scale>0.7</scale>
			<Icon>
				<href>http://maps.google.com/mapfiles/kml/paddle/wht-stars.png</href>
			</Icon>
			<hotSpot x="32" y="1" xunits="pixels" yunits="pixels"/>
		</IconStyle>
	</Style>
	<Style id="sn_wht-stars">
		<IconStyle>
			<scale>0.5</scale>
			<Icon>
				<href>http://maps.google.com/mapfiles/kml/paddle/wht-stars.png</href>
			</Icon>
			<hotSpot x="32" y="1" xunits="pixels" yunits="pixels"/>
		</IconStyle>
		<LabelStyle>
			<color>00ffffff</color>
		</LabelStyle>
	</Style>
	<Style id="sh_ylw-stars">
		<IconStyle>
			<scale>0.7</scale>
			<Icon>
				<href>http://maps.google.com/mapfiles/kml/paddle/ylw-stars.png</href>
			</Icon>
			<hotSpot x="32" y="1" xunits="pixels" yunits="pixels"/>
		</IconStyle>
		<LabelStyle>
			<color>ff00ffff</color>
		</LabelStyle>
	</Style>
	<Style id="sn_ylw-stars">
		<IconStyle>
			<scale>0.5</scale>
			<Icon>
				<href>http://maps.google.com/mapfiles/kml/paddle/ylw-stars.png</href>
			</Icon>
			<hotSpot x="32" y="1" xunits="pixels" yunits="pixels"/>
		</IconStyle>
		<LabelStyle>
			<color>0000ffff</color>
		</LabelStyle>
	</Style>
	<StyleMap id="msn_grn-stars">
		<Pair>
			<key>normal</key>
			<styleUrl>#sn_grn-stars</styleUrl>
		</Pair>
		<Pair>
			<key>highlight</key>
			<styleUrl>#sh_grn-stars</styleUrl>
		</Pair>
	</StyleMap>
	<StyleMap id="msn_blu-stars">
		<Pair>
			<key>normal</key>
			<styleUrl>#sn_blu-stars</styleUrl>
		</Pair>
		<Pair>
			<key>highlight</key>
			<styleUrl>#sh_blu-stars</styleUrl>
		</Pair>
	</StyleMap>
	<StyleMap id="msn_wht-stars">
		<Pair>
			<key>normal</key>
			<styleUrl>#sn_wht-stars</styleUrl>
		</Pair>
		<Pair>
			<key>highlight</key>
			<styleUrl>#sh_wht-stars</styleUrl>
		</Pair>
	</StyleMap>
	<StyleMap id="msn_ylw-stars">
		<Pair>
			<key>normal</key>
			<styleUrl>#sn_ylw-stars</styleUrl>
		</Pair>
		<Pair>
			<key>highlight</key>
			<styleUrl>#sh_ylw-stars</styleUrl>
		</Pair>
	</StyleMap>
'''% (Ward)
HPHeader='''<Folder>
<name>High Priests</name>
<open>1</open>
'''
ELHeader='''<Folder>
<name>Elders</name>
<open>1</open>
'''
PEHeader='''<Folder>
<name>Perspective Elders</name>
<open>1</open>
'''
NPHeader='''<Folder>
<name>No Priesthood</name>
<open>1</open>
'''
FolderFooter='</Folder>'
myFooter='</Document></kml>'
gError=[]
gRNV=["###################The following have requested no visits###################\n"]
kmlString=[] 
myResult=[]
myResult.append(myHeader)
HP=[]
HP.append(HPHeader)
EL=[]
EL.append(ELHeader)
PE=[]
PE.append(PEHeader)
NP=[]
NP.append(NPHeader)
lat,lng = 0.0,0.0

num_entries = 0

while 1:

	# get data for this record
	try: row = rows.next()
	except: break
	if row[5]== "Head of Household":
		#time.sleep(1.8) #Pause between each google query - Google sometimes blocks people who query their service more than once per 1.725 seconds
		#Strip leading and trailing white space
		for i in xrange(len(row)):
			row[i]=row[i].strip()
		mrn=row[3]
		name=row[2]
		addr=row[10]
		city=row[13]
		state=row[15]
		country=row[16]
		email= row[9]
		phone1=row[7]
		phone2=row[8]
		postal=row[14]
		birth=row[27]
		Priesthood=row[32]
		flat=''
		entry_opened = 0
		num_entries = num_entries + 1
	
	
		#Get Prefered Name and Split With Comma
		splitName = name.split(',')
		lastName=splitName[0].strip()
		firstName=splitName[1].strip()
		#Get address and split with delimiters - , / \
		
		splitAddr=re.split(r"[\-\,\/\\]+",addr)
		#Remove white space before and after address entry
		for i in xrange(len(splitAddr)):
			splitAddr[i]=splitAddr[i].lstrip()
			splitAddr[i]=splitAddr[i].rstrip()
		
		if len(splitAddr) == 1:
			street=splitAddr[0]
		
		elif len(splitAddr) == 2:
			street=splitAddr[0]
			flat=splitAddr[1]
		elif len(splitAddr) == 3:
			street=splitAddr[0]
			flat=(splitAddr[1] + " " + splitAddr[2])
		elif len(splitAddr) == 4:
			street=splitAddr[0]
			flat=(splitAddr[1] + " " + splitAddr[2] + " " + splitAddr[3])	
		else:
			print ("I can't inrerpret the address for " + name + " there are too many delimiters.")
			continue
		
		#Get email and split with comma --- Currently unused but i'll do it just incase I decide to add it later.
		
		emailSplit=email.split(',')
		secondaryEmail=""
		if len(emailSplit) == 2:
			secondaryEmail=emailSplit[1]
		primaryEmail=emailSplit[0]
		
		#Write values for csv file to a string to prepare for writing.
		#rowX=[street + "," + flat + "," + city + "," + state + "," + country ]+[firstName + " " + lastName]

		
		#write the row to the file.
		#writer.writerow(rowX)
		
		#Get ready for KML processing
		myName=firstName +" "+ lastName
		url = ''
		lat = 0.0
		lng = 0.0

	
		# Skip loop if no street address is available, add line to error log
		splitAddr = addr.split(',')
		if flat != "":
			kmlAddr = street+", "+flat
			#print "street and flat address is " + kmlAddr
		else:
			kmlAddr = street
			#print "no Flat address exists"
		rowX=[firstName, lastName]+[phone1]+[phone2]+[street]+[flat]+[city]+[state]+[postal]+[country]+[primaryEmail, secondaryEmail]+[birth]+[lat]+[lng]+[mrn]
		
		if street=='':
			print ("# "+ myName +" doesn't have a street address - " + street + "\n")
			gError.append ("# "+ myName +" doesn't have a street address. \n\n")
			writer.writerow(rowX)
			continue
		
		# Skip loop if address inclueds "DNC" or Do Not Contact, add line to error log
		if street.find("DNC")>= 0:
			print ("# "+ myName +" Requested Do Not Contact(DNC) - omitting from list - " + street)
			gRNV.append("# "+ myName +" Requested Do Not Contact (DNC) - omitting from list\n")
			writer.writerow(rowX)
			continue
		
		#Use existing data
		if addressX.get((mrn))== street and latitudeX[(mrn)] != '0.0' and longitudeX[(mrn)] != '0.0':
			lat,lng=(latitudeX[(mrn)]),(longitudeX[(mrn)])
			#postal=(postalX[(mrn)])
			url = 'http://maps.google.com/maps?q=%s+%s' % (lat,lng)
			used=' - used existing data.'
			#' - used existing data.'

		#Run script with Google API key	
		elif key!='': 
			url = 'http://maps.google.com/maps/geo?q=' + urllib.quote(street + ", " + city + ", " + state + ", " + country) + '&output=kml&key='+key
			xml = urllib.urlopen(url).read()
			code = xml[xml.find('<code>')+6:xml.find('</',xml.find('<code>'))]
			if code!="200":
				print("#Error Code:" +code+ " Google cannot interpret the address for " + myName)
				gError.append("#Error Code:" +code+ " Google cannot interpret the address for " + myName+"\n")
				writer.writerow(rowX)
				continue
			#postal = xml[xml.find('<PostalCodeNumber>')+18:xml.find('</',xml.find('<PostalCodeNumber>'))]
			coordinates = xml[xml.find('<coordinates>')+13:xml.find('</',xml.find('<coordinates>'))]
			lng,lat,num = coordinates.split(',')
		#If key is not available use manual method
		else:
			url = 'http://maps.google.com/?q=' + urllib.quote(street + ", " + city + ", " + state + ", " + country) + '&output=js'
			xml = urllib.urlopen(url).read()
			# Skip loop if google url contains the word error.
			if '<error>'in xml:
				print '# Google cannot interpret the address.\n'
				gError.append("# Google cannot interpret the address for " + myName +"\n" + street + flat + ";" + myName + "\n\n")
				writer.writerow(rowX)
				continue
			center = xml[xml.find('{center')+10:xml.find('}',xml.find('{center'))]
			center = center.replace('lat:','').replace('lng:','').replace('at:','')
			lat,lng = center.split(',')
			#for some unknown(to me) reason google gives the latitude 37.0625 for addresses it cant interpret, cull these out and add to error log.
			if lat=="37.0625":
				print ('# Google cannot interpret the address for ' + myName +" "+ addr)
				gError.append('# Google cannot interpret the address for ' + myName +"\n" + addr + ";" +myName + "\n\n")
				writer.writerow(rowX)
				continue 

                # Define Icon and Folder based on priesthood of HoH
		if Priesthood=="High Priest" and url <>'':
                        dataToSave='\t<Placemark>\n\t\t<name>%s</name>\n\t\t<description><![CDATA[%s<br>%s<br>%s<br>%s<br>%s]]></description>\n\t\t<styleUrl>#msn_grn-stars</styleUrl>\n\t\t<Point>\n\t\t<coordinates>%s,%s</coordinates>\n\t\t</Point>\n\t</Placemark>'% (myName,kmlAddr,city,state,phone1,phone2,lng,lat)
			#kmlString = kmlString + [dataToSave]
			HP.append(dataToSave)
			HP.append("\n")
			printMe='[%s] lat : %s, long : %s, - %s' % (num_entries,lat,lng, myName) + used
			print printMe
		elif Priesthood=="Elder" and url <>'':
                        dataToSave='\t<Placemark>\n\t\t<name>%s</name>\n\t\t<description><![CDATA[%s<br>%s<br>%s<br>%s<br>%s]]></description>\n\t\t<styleUrl>#msn_blu-stars</styleUrl>\n\t\t<Point>\n\t\t<coordinates>%s,%s</coordinates>\n\t\t</Point>\n\t</Placemark>'% (myName,kmlAddr,city,state,phone1,phone2,lng,lat)
			#kmlString = kmlString + [dataToSave]
			EL.append(dataToSave)
			EL.append("\n")
			printMe='[%s] lat : %s, long : %s, - %s' % (num_entries,lat,lng, myName) + used
			print printMe
                elif Priesthood==''or Priesthood=="Unordained" and url <>'':
                        dataToSave='\t<Placemark>\n\t\t<name>%s</name>\n\t\t<description><![CDATA[%s<br>%s<br>%s<br>%s<br>%s]]></description>\n\t\t<styleUrl>#msn_wht-stars</styleUrl>\n\t\t<Point>\n\t\t<coordinates>%s,%s</coordinates>\n\t\t</Point>\n\t</Placemark>'% (myName,kmlAddr,city,state,phone1,phone2,lng,lat)
			#kmlString = kmlString + [dataToSave]
			NP.append(dataToSave)
			NP.append("\n")
			printMe='[%s] lat : %s, long : %s, - %s' % (num_entries,lat,lng, myName) + used
			print printMe
                elif url <>'':
                        dataToSave='\t<Placemark>\n\t\t<name>%s</name>\n\t\t<description><![CDATA[%s<br>%s<br>%s<br>%s<br>%s]]></description>\n\t\t<styleUrl>#msn_ylw-stars</styleUrl>\n\t\t<Point>\n\t\t<coordinates>%s,%s</coordinates>\n\t\t</Point>\n\t</Placemark>'% (myName,kmlAddr,city,state,phone1,phone2,lng,lat)
			#kmlString = kmlString + [dataToSave]
			PE.append(dataToSave)
			PE.append("\n")
			printMe='[%s] lat : %s, long : %s, - %s' % (num_entries,lat,lng, myName) + used
			print printMe
		
		#The following is the full information string that you may wish to include in an address book.
		rowX=[firstName, lastName]+[phone1]+[phone2]+[street]+[flat]+[city]+[state]+[postal]+[country]+[primaryEmail, secondaryEmail]+[birth]+[lat]+[lng]+[mrn]
		writer.writerow(rowX)
		
'''
		
	except IndexError:
		print 'IndexError'
		pass 
	except ValueError:
		print xml
		print 'ValueError'
		pass
		
'''	
HP.append(FolderFooter)
EL.append(FolderFooter)
PE.append(FolderFooter)
NP.append(FolderFooter)
myResult.extend(HP)
myResult.extend(EL)
myResult.extend(PE)
myResult.extend(NP)
myResult.append(myFooter)
gRNV.append("###########################################################################\n\n")
# Restore STDOUT and close the files
#sys.stdout = real_stdout

f.close()
#kmlFile.writelines(writer)
memReformat.close()

# print myResult

kmlFile = open((''' %s_wardMap.kml''' % (Ward)), 'w')
kmlFile.writelines(myResult)
kmlFile.close()
fError = open(('geoError.txt'), 'w')
fError.writelines(gRNV)
fError.writelines(gError)
fError.close()


'''
print
print 'Done! A total of %d contacts were read.' % num_entries
print 'The resulting CSV file was saved as', outfile
print 'The resulting KML file was saved as', kmlFile
print 'The resulting Error log was saved as', fError
print 
'''
